@@ -4,7 +4,7 @@ context("cities")
 #################################################################################################
 test_that("cities returns a data.frame (tbl_df)", {
   skip_on_cran()
-  output <- aq_cities()
+  output <- aq_cities(page = 1)
   expect_that(output, is_a("tbl_df"))
   expect_that(attr(output, "meta"), is_a("tbl_df"))
   expect_that(attr(output, "timestamp"), is_a("tbl_df"))
@@ -13,7 +13,7 @@ test_that("cities returns a data.frame (tbl_df)", {
 
 test_that("cities has the right columns", {
   skip_on_cran()
-  tableRes <- aq_cities(country="IN")
+  tableRes <- aq_cities(page = 1, country="IN")
   expect_true(class(tableRes$city) == "character")
   expect_true(class(tableRes$country) == "character")
   expect_true(class(tableRes$cityURL) == "character")
@@ -23,7 +23,6 @@ test_that("cities has the right columns", {
   expect_true(all(names(meta) == c("name", "license",
                                    "website", "page",
                                    "limit", "found")))
-  expect_is(attr(tableRes, "timestamp")$lastModif, "POSIXt")
   expect_is(attr(tableRes, "timestamp")$queriedAt, "POSIXt")
 
 })
